from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="Metadata")


@_attrs_define
class Metadata:
    """
    Attributes:
        process_url_in_text (bool | Unset):  Default: False.
        process_attachment_url (bool | Unset):  Default: True.
        attachment_requested (int | Unset):  Default: 0.
    """

    process_url_in_text: bool | Unset = False
    process_attachment_url: bool | Unset = True
    attachment_requested: int | Unset = 0
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        process_url_in_text = self.process_url_in_text

        process_attachment_url = self.process_attachment_url

        attachment_requested = self.attachment_requested

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if process_url_in_text is not UNSET:
            field_dict["process_url_in_text"] = process_url_in_text
        if process_attachment_url is not UNSET:
            field_dict["process_attachment_url"] = process_attachment_url
        if attachment_requested is not UNSET:
            field_dict["attachment_requested"] = attachment_requested

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        process_url_in_text = d.pop("process_url_in_text", UNSET)

        process_attachment_url = d.pop("process_attachment_url", UNSET)

        attachment_requested = d.pop("attachment_requested", UNSET)

        metadata = cls(
            process_url_in_text=process_url_in_text,
            process_attachment_url=process_attachment_url,
            attachment_requested=attachment_requested,
        )

        metadata.additional_properties = d
        return metadata

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="LookerClientCredentialsRequest")


@_attrs_define
class LookerClientCredentialsRequest:
    """
    Attributes:
        base_url (str):
        client_id (str):
        client_secret (str):
    """

    base_url: str
    client_id: str
    client_secret: str
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        base_url = self.base_url

        client_id = self.client_id

        client_secret = self.client_secret

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "base_url": base_url,
                "client_id": client_id,
                "client_secret": client_secret,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        base_url = d.pop("base_url")

        client_id = d.pop("client_id")

        client_secret = d.pop("client_secret")

        looker_client_credentials_request = cls(
            base_url=base_url,
            client_id=client_id,
            client_secret=client_secret,
        )

        looker_client_credentials_request.additional_properties = d
        return looker_client_credentials_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.db_auth_lite_doc import DbAuthLiteDoc


T = TypeVar("T", bound="DbAuthLiteDocPage")


@_attrs_define
class DbAuthLiteDocPage:
    """
    Attributes:
        docs (list[DbAuthLiteDoc]):
        total_count (int):
        limit (int):
        offset (int):
    """

    docs: list[DbAuthLiteDoc]
    total_count: int
    limit: int
    offset: int
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        docs = []
        for docs_item_data in self.docs:
            docs_item = docs_item_data.to_dict()
            docs.append(docs_item)

        total_count = self.total_count

        limit = self.limit

        offset = self.offset

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "docs": docs,
                "total_count": total_count,
                "limit": limit,
                "offset": offset,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.db_auth_lite_doc import DbAuthLiteDoc

        d = dict(src_dict)
        docs = []
        _docs = d.pop("docs")
        for docs_item_data in _docs:
            docs_item = DbAuthLiteDoc.from_dict(docs_item_data)

            docs.append(docs_item)

        total_count = d.pop("total_count")

        limit = d.pop("limit")

        offset = d.pop("offset")

        db_auth_lite_doc_page = cls(
            docs=docs,
            total_count=total_count,
            limit=limit,
            offset=offset,
        )

        db_auth_lite_doc_page.additional_properties = d
        return db_auth_lite_doc_page

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class EventType(str, Enum):
    ACTION_GRAPH_RUN = "action_graph_run"
    ACTION_RUN = "action_run"
    DATASTORE_PUT = "datastore_put"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.filter_by import FilterBy
from ..models.time_range import TimeRange
from ..types import UNSET, Unset

T = TypeVar("T", bound="DashboardRequest")


@_attrs_define
class DashboardRequest:
    """Request model for dashboard data

    Attributes:
        time_range (TimeRange | Unset):
        start_date (datetime.datetime | None | Unset):
        end_date (datetime.datetime | None | Unset):
        filter_by (FilterBy | None | Unset):
        filter_value (None | str | Unset):
    """

    time_range: TimeRange | Unset = UNSET
    start_date: datetime.datetime | None | Unset = UNSET
    end_date: datetime.datetime | None | Unset = UNSET
    filter_by: FilterBy | None | Unset = UNSET
    filter_value: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        time_range: str | Unset = UNSET
        if not isinstance(self.time_range, Unset):
            time_range = self.time_range.value

        start_date: None | str | Unset
        if isinstance(self.start_date, Unset):
            start_date = UNSET
        elif isinstance(self.start_date, datetime.datetime):
            start_date = self.start_date.isoformat()
        else:
            start_date = self.start_date

        end_date: None | str | Unset
        if isinstance(self.end_date, Unset):
            end_date = UNSET
        elif isinstance(self.end_date, datetime.datetime):
            end_date = self.end_date.isoformat()
        else:
            end_date = self.end_date

        filter_by: None | str | Unset
        if isinstance(self.filter_by, Unset):
            filter_by = UNSET
        elif isinstance(self.filter_by, FilterBy):
            filter_by = self.filter_by.value
        else:
            filter_by = self.filter_by

        filter_value: None | str | Unset
        if isinstance(self.filter_value, Unset):
            filter_value = UNSET
        else:
            filter_value = self.filter_value

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if time_range is not UNSET:
            field_dict["time_range"] = time_range
        if start_date is not UNSET:
            field_dict["start_date"] = start_date
        if end_date is not UNSET:
            field_dict["end_date"] = end_date
        if filter_by is not UNSET:
            field_dict["filter_by"] = filter_by
        if filter_value is not UNSET:
            field_dict["filter_value"] = filter_value

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        _time_range = d.pop("time_range", UNSET)
        time_range: TimeRange | Unset
        if isinstance(_time_range, Unset):
            time_range = UNSET
        else:
            time_range = TimeRange(_time_range)

        def _parse_start_date(data: object) -> datetime.datetime | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                start_date_type_0 = isoparse(data)

                return start_date_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(datetime.datetime | None | Unset, data)

        start_date = _parse_start_date(d.pop("start_date", UNSET))

        def _parse_end_date(data: object) -> datetime.datetime | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                end_date_type_0 = isoparse(data)

                return end_date_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(datetime.datetime | None | Unset, data)

        end_date = _parse_end_date(d.pop("end_date", UNSET))

        def _parse_filter_by(data: object) -> FilterBy | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                filter_by_type_0 = FilterBy(data)

                return filter_by_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(FilterBy | None | Unset, data)

        filter_by = _parse_filter_by(d.pop("filter_by", UNSET))

        def _parse_filter_value(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        filter_value = _parse_filter_value(d.pop("filter_value", UNSET))

        dashboard_request = cls(
            time_range=time_range,
            start_date=start_date,
            end_date=end_date,
            filter_by=filter_by,
            filter_value=filter_value,
        )

        dashboard_request.additional_properties = d
        return dashboard_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="StaticFileChunkingStrategyParam")


@_attrs_define
class StaticFileChunkingStrategyParam:
    """
    Attributes:
        chunk_overlap_tokens (int):
        max_chunk_size_tokens (int):
    """

    chunk_overlap_tokens: int
    max_chunk_size_tokens: int
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        chunk_overlap_tokens = self.chunk_overlap_tokens

        max_chunk_size_tokens = self.max_chunk_size_tokens

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "chunk_overlap_tokens": chunk_overlap_tokens,
                "max_chunk_size_tokens": max_chunk_size_tokens,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        chunk_overlap_tokens = d.pop("chunk_overlap_tokens")

        max_chunk_size_tokens = d.pop("max_chunk_size_tokens")

        static_file_chunking_strategy_param = cls(
            chunk_overlap_tokens=chunk_overlap_tokens,
            max_chunk_size_tokens=max_chunk_size_tokens,
        )

        static_file_chunking_strategy_param.additional_properties = d
        return static_file_chunking_strategy_param

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class DatabaseEnum(str, Enum):
    BIGQUERY = "bigquery"
    MYSQL = "mysql"
    ORACLE = "oracle"
    POSTGRES = "postgres"
    SQLITE = "sqlite"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class GetVectorStoreV1OpenaiStorageVectorStoresGetOpenaiProvider(str, Enum):
    AZURE = "azure"
    OPENAI = "openai"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="JobStatusSchema")


@_attrs_define
class JobStatusSchema:
    """
    Attributes:
        field_id (str):
        user_id (str):
        status (str):
        weaviate_status (str):
        graphrag_status (str):
        created_at (datetime.datetime):
        updated_at (datetime.datetime):
        kb_id (None | str | Unset):
        tenant_id (None | str | Unset):
        uploaded_by_user_id (None | str | Unset):
        uploaded_by_email (None | str | Unset):
        collection_name (None | str | Unset):
        original_filename (None | str | Unset):
        s3_url (None | str | Unset):
        error_message (None | str | Unset):
    """

    field_id: str
    user_id: str
    status: str
    weaviate_status: str
    graphrag_status: str
    created_at: datetime.datetime
    updated_at: datetime.datetime
    kb_id: None | str | Unset = UNSET
    tenant_id: None | str | Unset = UNSET
    uploaded_by_user_id: None | str | Unset = UNSET
    uploaded_by_email: None | str | Unset = UNSET
    collection_name: None | str | Unset = UNSET
    original_filename: None | str | Unset = UNSET
    s3_url: None | str | Unset = UNSET
    error_message: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        field_id = self.field_id

        user_id = self.user_id

        status = self.status

        weaviate_status = self.weaviate_status

        graphrag_status = self.graphrag_status

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        kb_id: None | str | Unset
        if isinstance(self.kb_id, Unset):
            kb_id = UNSET
        else:
            kb_id = self.kb_id

        tenant_id: None | str | Unset
        if isinstance(self.tenant_id, Unset):
            tenant_id = UNSET
        else:
            tenant_id = self.tenant_id

        uploaded_by_user_id: None | str | Unset
        if isinstance(self.uploaded_by_user_id, Unset):
            uploaded_by_user_id = UNSET
        else:
            uploaded_by_user_id = self.uploaded_by_user_id

        uploaded_by_email: None | str | Unset
        if isinstance(self.uploaded_by_email, Unset):
            uploaded_by_email = UNSET
        else:
            uploaded_by_email = self.uploaded_by_email

        collection_name: None | str | Unset
        if isinstance(self.collection_name, Unset):
            collection_name = UNSET
        else:
            collection_name = self.collection_name

        original_filename: None | str | Unset
        if isinstance(self.original_filename, Unset):
            original_filename = UNSET
        else:
            original_filename = self.original_filename

        s3_url: None | str | Unset
        if isinstance(self.s3_url, Unset):
            s3_url = UNSET
        else:
            s3_url = self.s3_url

        error_message: None | str | Unset
        if isinstance(self.error_message, Unset):
            error_message = UNSET
        else:
            error_message = self.error_message

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "_id": field_id,
                "user_id": user_id,
                "status": status,
                "weaviate_status": weaviate_status,
                "graphrag_status": graphrag_status,
                "created_at": created_at,
                "updated_at": updated_at,
            }
        )
        if kb_id is not UNSET:
            field_dict["kb_id"] = kb_id
        if tenant_id is not UNSET:
            field_dict["tenant_id"] = tenant_id
        if uploaded_by_user_id is not UNSET:
            field_dict["uploaded_by_user_id"] = uploaded_by_user_id
        if uploaded_by_email is not UNSET:
            field_dict["uploaded_by_email"] = uploaded_by_email
        if collection_name is not UNSET:
            field_dict["collection_name"] = collection_name
        if original_filename is not UNSET:
            field_dict["original_filename"] = original_filename
        if s3_url is not UNSET:
            field_dict["s3_url"] = s3_url
        if error_message is not UNSET:
            field_dict["error_message"] = error_message

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        field_id = d.pop("_id")

        user_id = d.pop("user_id")

        status = d.pop("status")

        weaviate_status = d.pop("weaviate_status")

        graphrag_status = d.pop("graphrag_status")

        created_at = isoparse(d.pop("created_at"))

        updated_at = isoparse(d.pop("updated_at"))

        def _parse_kb_id(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        kb_id = _parse_kb_id(d.pop("kb_id", UNSET))

        def _parse_tenant_id(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        tenant_id = _parse_tenant_id(d.pop("tenant_id", UNSET))

        def _parse_uploaded_by_user_id(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        uploaded_by_user_id = _parse_uploaded_by_user_id(d.pop("uploaded_by_user_id", UNSET))

        def _parse_uploaded_by_email(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        uploaded_by_email = _parse_uploaded_by_email(d.pop("uploaded_by_email", UNSET))

        def _parse_collection_name(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        collection_name = _parse_collection_name(d.pop("collection_name", UNSET))

        def _parse_original_filename(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        original_filename = _parse_original_filename(d.pop("original_filename", UNSET))

        def _parse_s3_url(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        s3_url = _parse_s3_url(d.pop("s3_url", UNSET))

        def _parse_error_message(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        error_message = _parse_error_message(d.pop("error_message", UNSET))

        job_status_schema = cls(
            field_id=field_id,
            user_id=user_id,
            status=status,
            weaviate_status=weaviate_status,
            graphrag_status=graphrag_status,
            created_at=created_at,
            updated_at=updated_at,
            kb_id=kb_id,
            tenant_id=tenant_id,
            uploaded_by_user_id=uploaded_by_user_id,
            uploaded_by_email=uploaded_by_email,
            collection_name=collection_name,
            original_filename=original_filename,
            s3_url=s3_url,
            error_message=error_message,
        )

        job_status_schema.additional_properties = d
        return job_status_schema

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.node_data import NodeData
    from ..models.position import Position


T = TypeVar("T", bound="Node")


@_attrs_define
class Node:
    """
    Attributes:
        id (str):
        data (NodeData):
        position (None | Position | Unset):
        type_ (None | str | Unset):
    """

    id: str
    data: NodeData
    position: None | Position | Unset = UNSET
    type_: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.position import Position

        id = self.id

        data = self.data.to_dict()

        position: dict[str, Any] | None | Unset
        if isinstance(self.position, Unset):
            position = UNSET
        elif isinstance(self.position, Position):
            position = self.position.to_dict()
        else:
            position = self.position

        type_: None | str | Unset
        if isinstance(self.type_, Unset):
            type_ = UNSET
        else:
            type_ = self.type_

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "data": data,
            }
        )
        if position is not UNSET:
            field_dict["position"] = position
        if type_ is not UNSET:
            field_dict["type"] = type_

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.node_data import NodeData
        from ..models.position import Position

        d = dict(src_dict)
        id = d.pop("id")

        data = NodeData.from_dict(d.pop("data"))

        def _parse_position(data: object) -> None | Position | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                position_type_0 = Position.from_dict(data)

                return position_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Position | Unset, data)

        position = _parse_position(d.pop("position", UNSET))

        def _parse_type_(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        type_ = _parse_type_(d.pop("type", UNSET))

        node = cls(
            id=id,
            data=data,
            position=position,
            type_=type_,
        )

        node.additional_properties = d
        return node

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.llm_chat_openai_config_modalities_type_0_item import LlmChatOpenaiConfigModalitiesType0Item
from ..models.llm_chat_openai_config_model_type_0 import LlmChatOpenaiConfigModelType0
from ..models.llm_chat_openai_config_model_type_1 import LlmChatOpenaiConfigModelType1
from ..models.llm_chat_openai_config_prompt_cache_retention_type_0 import LlmChatOpenaiConfigPromptCacheRetentionType0
from ..models.llm_chat_openai_config_provider import LlmChatOpenaiConfigProvider
from ..models.llm_chat_openai_config_reasoning_effort_type_0 import LlmChatOpenaiConfigReasoningEffortType0
from ..models.llm_chat_openai_config_service_tier_type_0 import LlmChatOpenaiConfigServiceTierType0
from ..models.llm_chat_openai_config_tool_choice_type_0 import LlmChatOpenaiConfigToolChoiceType0
from ..models.llm_chat_openai_config_verbosity_type_0 import LlmChatOpenaiConfigVerbosityType0
from ..models.output_type import OutputType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.chat_completion_allowed_tool_choice_param import ChatCompletionAllowedToolChoiceParam
    from ..models.chat_completion_assistant_message_param import ChatCompletionAssistantMessageParam
    from ..models.chat_completion_audio_param import ChatCompletionAudioParam
    from ..models.chat_completion_developer_message_param import ChatCompletionDeveloperMessageParam
    from ..models.chat_completion_function_message_param import ChatCompletionFunctionMessageParam
    from ..models.chat_completion_function_tool_param import ChatCompletionFunctionToolParam
    from ..models.chat_completion_named_tool_choice_custom_param import ChatCompletionNamedToolChoiceCustomParam
    from ..models.chat_completion_named_tool_choice_param import ChatCompletionNamedToolChoiceParam
    from ..models.chat_completion_prediction_content_param import ChatCompletionPredictionContentParam
    from ..models.chat_completion_stream_options_param import ChatCompletionStreamOptionsParam
    from ..models.chat_completion_system_message_param import ChatCompletionSystemMessageParam
    from ..models.chat_completion_tool_message_param import ChatCompletionToolMessageParam
    from ..models.chat_completion_user_message_param import ChatCompletionUserMessageParam
    from ..models.llm_chat_openai_config_extra_headers_type_0 import LlmChatOpenaiConfigExtraHeadersType0
    from ..models.llm_chat_openai_config_extra_query_type_0 import LlmChatOpenaiConfigExtraQueryType0
    from ..models.llm_chat_openai_config_json_schema_type_0 import LlmChatOpenaiConfigJsonSchemaType0
    from ..models.llm_chat_openai_config_logit_bias_type_0 import LlmChatOpenaiConfigLogitBiasType0
    from ..models.llm_chat_openai_config_metadata_type_0 import LlmChatOpenaiConfigMetadataType0
    from ..models.llm_chat_openai_config_response_schema_type_1 import LlmChatOpenaiConfigResponseSchemaType1
    from ..models.response_format_json_object import ResponseFormatJSONObject
    from ..models.response_format_json_schema import ResponseFormatJSONSchema
    from ..models.response_format_text import ResponseFormatText
    from ..models.structured_output_schema import StructuredOutputSchema
    from ..models.web_search_options import WebSearchOptions


T = TypeVar("T", bound="LlmChatOpenaiConfig")


@_attrs_define
class LlmChatOpenaiConfig:
    """
    Attributes:
        messages (list[ChatCompletionAssistantMessageParam | ChatCompletionDeveloperMessageParam |
            ChatCompletionFunctionMessageParam | ChatCompletionSystemMessageParam | ChatCompletionToolMessageParam |
            ChatCompletionUserMessageParam]):
        provider (LlmChatOpenaiConfigProvider):
        model (LlmChatOpenaiConfigModelType0 | LlmChatOpenaiConfigModelType1 | str | Unset):  Default:
            LlmChatOpenaiConfigModelType0.GPT_4_1_MINI.
        audio (ChatCompletionAudioParam | None | Unset):
        frequency_penalty (float | None | Unset):
        logit_bias (LlmChatOpenaiConfigLogitBiasType0 | None | Unset):
        logprobs (bool | None | Unset):
        max_completion_tokens (int | None | Unset):
        max_tokens (int | None | Unset):
        metadata (LlmChatOpenaiConfigMetadataType0 | None | Unset):
        modalities (list[LlmChatOpenaiConfigModalitiesType0Item] | None | Unset):
        n (int | None | Unset):  Default: 1.
        parallel_tool_calls (bool | None | Unset):
        prediction (ChatCompletionPredictionContentParam | None | Unset):
        presence_penalty (float | None | Unset):
        prompt_cache_key (None | str | Unset):
        prompt_cache_retention (LlmChatOpenaiConfigPromptCacheRetentionType0 | None | Unset):
        reasoning_effort (LlmChatOpenaiConfigReasoningEffortType0 | None | Unset):
        response_format (None | ResponseFormatJSONObject | ResponseFormatJSONSchema | ResponseFormatText | Unset):
        safety_identifier (None | str | Unset):
        seed (int | None | Unset):
        service_tier (LlmChatOpenaiConfigServiceTierType0 | None | Unset):
        stop (list[str] | None | str | Unset):
        store (bool | None | Unset):
        stream (bool | None | Unset):  Default: False.
        stream_options (ChatCompletionStreamOptionsParam | None | Unset):
        temperature (float | None | Unset):
        tool_choice (ChatCompletionAllowedToolChoiceParam | ChatCompletionNamedToolChoiceCustomParam |
            ChatCompletionNamedToolChoiceParam | LlmChatOpenaiConfigToolChoiceType0 | None | Unset):
        tools (list[ChatCompletionFunctionToolParam | str] | None | Unset):
        top_logprobs (int | None | Unset):
        top_p (float | None | Unset):
        user (None | str | Unset):
        verbosity (LlmChatOpenaiConfigVerbosityType0 | None | Unset):
        web_search_options (None | Unset | WebSearchOptions):
        output_type (None | OutputType | Unset):
        json_schema (LlmChatOpenaiConfigJsonSchemaType0 | None | Unset):
        response_schema (LlmChatOpenaiConfigResponseSchemaType1 | None | StructuredOutputSchema | Unset):
        extra_headers (LlmChatOpenaiConfigExtraHeadersType0 | None | Unset):
        extra_query (LlmChatOpenaiConfigExtraQueryType0 | None | Unset):
        extra_body (Any | None | Unset):
        timeout (float | None | Unset):  Default: 900.0.
        tool_output_text_to_file_char_limit (int | None | Unset):  Default: 75000.
    """

    messages: list[
        ChatCompletionAssistantMessageParam
        | ChatCompletionDeveloperMessageParam
        | ChatCompletionFunctionMessageParam
        | ChatCompletionSystemMessageParam
        | ChatCompletionToolMessageParam
        | ChatCompletionUserMessageParam
    ]
    provider: LlmChatOpenaiConfigProvider
    model: LlmChatOpenaiConfigModelType0 | LlmChatOpenaiConfigModelType1 | str | Unset = (
        LlmChatOpenaiConfigModelType0.GPT_4_1_MINI
    )
    audio: ChatCompletionAudioParam | None | Unset = UNSET
    frequency_penalty: float | None | Unset = UNSET
    logit_bias: LlmChatOpenaiConfigLogitBiasType0 | None | Unset = UNSET
    logprobs: bool | None | Unset = UNSET
    max_completion_tokens: int | None | Unset = UNSET
    max_tokens: int | None | Unset = UNSET
    metadata: LlmChatOpenaiConfigMetadataType0 | None | Unset = UNSET
    modalities: list[LlmChatOpenaiConfigModalitiesType0Item] | None | Unset = UNSET
    n: int | None | Unset = 1
    parallel_tool_calls: bool | None | Unset = UNSET
    prediction: ChatCompletionPredictionContentParam | None | Unset = UNSET
    presence_penalty: float | None | Unset = UNSET
    prompt_cache_key: None | str | Unset = UNSET
    prompt_cache_retention: LlmChatOpenaiConfigPromptCacheRetentionType0 | None | Unset = UNSET
    reasoning_effort: LlmChatOpenaiConfigReasoningEffortType0 | None | Unset = UNSET
    response_format: None | ResponseFormatJSONObject | ResponseFormatJSONSchema | ResponseFormatText | Unset = UNSET
    safety_identifier: None | str | Unset = UNSET
    seed: int | None | Unset = UNSET
    service_tier: LlmChatOpenaiConfigServiceTierType0 | None | Unset = UNSET
    stop: list[str] | None | str | Unset = UNSET
    store: bool | None | Unset = UNSET
    stream: bool | None | Unset = False
    stream_options: ChatCompletionStreamOptionsParam | None | Unset = UNSET
    temperature: float | None | Unset = UNSET
    tool_choice: (
        ChatCompletionAllowedToolChoiceParam
        | ChatCompletionNamedToolChoiceCustomParam
        | ChatCompletionNamedToolChoiceParam
        | LlmChatOpenaiConfigToolChoiceType0
        | None
        | Unset
    ) = UNSET
    tools: list[ChatCompletionFunctionToolParam | str] | None | Unset = UNSET
    top_logprobs: int | None | Unset = UNSET
    top_p: float | None | Unset = UNSET
    user: None | str | Unset = UNSET
    verbosity: LlmChatOpenaiConfigVerbosityType0 | None | Unset = UNSET
    web_search_options: None | Unset | WebSearchOptions = UNSET
    output_type: None | OutputType | Unset = UNSET
    json_schema: LlmChatOpenaiConfigJsonSchemaType0 | None | Unset = UNSET
    response_schema: LlmChatOpenaiConfigResponseSchemaType1 | None | StructuredOutputSchema | Unset = UNSET
    extra_headers: LlmChatOpenaiConfigExtraHeadersType0 | None | Unset = UNSET
    extra_query: LlmChatOpenaiConfigExtraQueryType0 | None | Unset = UNSET
    extra_body: Any | None | Unset = UNSET
    timeout: float | None | Unset = 900.0
    tool_output_text_to_file_char_limit: int | None | Unset = 75000
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.chat_completion_allowed_tool_choice_param import ChatCompletionAllowedToolChoiceParam
        from ..models.chat_completion_assistant_message_param import ChatCompletionAssistantMessageParam
        from ..models.chat_completion_audio_param import ChatCompletionAudioParam
        from ..models.chat_completion_developer_message_param import ChatCompletionDeveloperMessageParam
        from ..models.chat_completion_function_tool_param import ChatCompletionFunctionToolParam
        from ..models.chat_completion_named_tool_choice_custom_param import ChatCompletionNamedToolChoiceCustomParam
        from ..models.chat_completion_named_tool_choice_param import ChatCompletionNamedToolChoiceParam
        from ..models.chat_completion_prediction_content_param import ChatCompletionPredictionContentParam
        from ..models.chat_completion_stream_options_param import ChatCompletionStreamOptionsParam
        from ..models.chat_completion_system_message_param import ChatCompletionSystemMessageParam
        from ..models.chat_completion_tool_message_param import ChatCompletionToolMessageParam
        from ..models.chat_completion_user_message_param import ChatCompletionUserMessageParam
        from ..models.llm_chat_openai_config_extra_headers_type_0 import LlmChatOpenaiConfigExtraHeadersType0
        from ..models.llm_chat_openai_config_extra_query_type_0 import LlmChatOpenaiConfigExtraQueryType0
        from ..models.llm_chat_openai_config_json_schema_type_0 import LlmChatOpenaiConfigJsonSchemaType0
        from ..models.llm_chat_openai_config_logit_bias_type_0 import LlmChatOpenaiConfigLogitBiasType0
        from ..models.llm_chat_openai_config_metadata_type_0 import LlmChatOpenaiConfigMetadataType0
        from ..models.llm_chat_openai_config_response_schema_type_1 import LlmChatOpenaiConfigResponseSchemaType1
        from ..models.response_format_json_object import ResponseFormatJSONObject
        from ..models.response_format_json_schema import ResponseFormatJSONSchema
        from ..models.response_format_text import ResponseFormatText
        from ..models.structured_output_schema import StructuredOutputSchema
        from ..models.web_search_options import WebSearchOptions

        messages = []
        for messages_item_data in self.messages:
            messages_item: dict[str, Any]
            if isinstance(messages_item_data, ChatCompletionDeveloperMessageParam):
                messages_item = messages_item_data.to_dict()
            elif isinstance(messages_item_data, ChatCompletionSystemMessageParam):
                messages_item = messages_item_data.to_dict()
            elif isinstance(messages_item_data, ChatCompletionUserMessageParam):
                messages_item = messages_item_data.to_dict()
            elif isinstance(messages_item_data, ChatCompletionAssistantMessageParam):
                messages_item = messages_item_data.to_dict()
            elif isinstance(messages_item_data, ChatCompletionToolMessageParam):
                messages_item = messages_item_data.to_dict()
            else:
                messages_item = messages_item_data.to_dict()

            messages.append(messages_item)

        provider = self.provider.value

        model: str | Unset
        if isinstance(self.model, Unset):
            model = UNSET
        elif isinstance(self.model, LlmChatOpenaiConfigModelType0):
            model = self.model.value
        elif isinstance(self.model, LlmChatOpenaiConfigModelType1):
            model = self.model.value
        else:
            model = self.model

        audio: dict[str, Any] | None | Unset
        if isinstance(self.audio, Unset):
            audio = UNSET
        elif isinstance(self.audio, ChatCompletionAudioParam):
            audio = self.audio.to_dict()
        else:
            audio = self.audio

        frequency_penalty: float | None | Unset
        if isinstance(self.frequency_penalty, Unset):
            frequency_penalty = UNSET
        else:
            frequency_penalty = self.frequency_penalty

        logit_bias: dict[str, Any] | None | Unset
        if isinstance(self.logit_bias, Unset):
            logit_bias = UNSET
        elif isinstance(self.logit_bias, LlmChatOpenaiConfigLogitBiasType0):
            logit_bias = self.logit_bias.to_dict()
        else:
            logit_bias = self.logit_bias

        logprobs: bool | None | Unset
        if isinstance(self.logprobs, Unset):
            logprobs = UNSET
        else:
            logprobs = self.logprobs

        max_completion_tokens: int | None | Unset
        if isinstance(self.max_completion_tokens, Unset):
            max_completion_tokens = UNSET
        else:
            max_completion_tokens = self.max_completion_tokens

        max_tokens: int | None | Unset
        if isinstance(self.max_tokens, Unset):
            max_tokens = UNSET
        else:
            max_tokens = self.max_tokens

        metadata: dict[str, Any] | None | Unset
        if isinstance(self.metadata, Unset):
            metadata = UNSET
        elif isinstance(self.metadata, LlmChatOpenaiConfigMetadataType0):
            metadata = self.metadata.to_dict()
        else:
            metadata = self.metadata

        modalities: list[str] | None | Unset
        if isinstance(self.modalities, Unset):
            modalities = UNSET
        elif isinstance(self.modalities, list):
            modalities = []
            for modalities_type_0_item_data in self.modalities:
                modalities_type_0_item = modalities_type_0_item_data.value
                modalities.append(modalities_type_0_item)

        else:
            modalities = self.modalities

        n: int | None | Unset
        if isinstance(self.n, Unset):
            n = UNSET
        else:
            n = self.n

        parallel_tool_calls: bool | None | Unset
        if isinstance(self.parallel_tool_calls, Unset):
            parallel_tool_calls = UNSET
        else:
            parallel_tool_calls = self.parallel_tool_calls

        prediction: dict[str, Any] | None | Unset
        if isinstance(self.prediction, Unset):
            prediction = UNSET
        elif isinstance(self.prediction, ChatCompletionPredictionContentParam):
            prediction = self.prediction.to_dict()
        else:
            prediction = self.prediction

        presence_penalty: float | None | Unset
        if isinstance(self.presence_penalty, Unset):
            presence_penalty = UNSET
        else:
            presence_penalty = self.presence_penalty

        prompt_cache_key: None | str | Unset
        if isinstance(self.prompt_cache_key, Unset):
            prompt_cache_key = UNSET
        else:
            prompt_cache_key = self.prompt_cache_key

        prompt_cache_retention: None | str | Unset
        if isinstance(self.prompt_cache_retention, Unset):
            prompt_cache_retention = UNSET
        elif isinstance(self.prompt_cache_retention, LlmChatOpenaiConfigPromptCacheRetentionType0):
            prompt_cache_retention = self.prompt_cache_retention.value
        else:
            prompt_cache_retention = self.prompt_cache_retention

        reasoning_effort: None | str | Unset
        if isinstance(self.reasoning_effort, Unset):
            reasoning_effort = UNSET
        elif isinstance(self.reasoning_effort, LlmChatOpenaiConfigReasoningEffortType0):
            reasoning_effort = self.reasoning_effort.value
        else:
            reasoning_effort = self.reasoning_effort

        response_format: dict[str, Any] | None | Unset
        if isinstance(self.response_format, Unset):
            response_format = UNSET
        elif isinstance(self.response_format, ResponseFormatText):
            response_format = self.response_format.to_dict()
        elif isinstance(self.response_format, ResponseFormatJSONSchema):
            response_format = self.response_format.to_dict()
        elif isinstance(self.response_format, ResponseFormatJSONObject):
            response_format = self.response_format.to_dict()
        else:
            response_format = self.response_format

        safety_identifier: None | str | Unset
        if isinstance(self.safety_identifier, Unset):
            safety_identifier = UNSET
        else:
            safety_identifier = self.safety_identifier

        seed: int | None | Unset
        if isinstance(self.seed, Unset):
            seed = UNSET
        else:
            seed = self.seed

        service_tier: None | str | Unset
        if isinstance(self.service_tier, Unset):
            service_tier = UNSET
        elif isinstance(self.service_tier, LlmChatOpenaiConfigServiceTierType0):
            service_tier = self.service_tier.value
        else:
            service_tier = self.service_tier

        stop: list[str] | None | str | Unset
        if isinstance(self.stop, Unset):
            stop = UNSET
        elif isinstance(self.stop, list):
            stop = self.stop

        else:
            stop = self.stop

        store: bool | None | Unset
        if isinstance(self.store, Unset):
            store = UNSET
        else:
            store = self.store

        stream: bool | None | Unset
        if isinstance(self.stream, Unset):
            stream = UNSET
        else:
            stream = self.stream

        stream_options: dict[str, Any] | None | Unset
        if isinstance(self.stream_options, Unset):
            stream_options = UNSET
        elif isinstance(self.stream_options, ChatCompletionStreamOptionsParam):
            stream_options = self.stream_options.to_dict()
        else:
            stream_options = self.stream_options

        temperature: float | None | Unset
        if isinstance(self.temperature, Unset):
            temperature = UNSET
        else:
            temperature = self.temperature

        tool_choice: dict[str, Any] | None | str | Unset
        if isinstance(self.tool_choice, Unset):
            tool_choice = UNSET
        elif isinstance(self.tool_choice, LlmChatOpenaiConfigToolChoiceType0):
            tool_choice = self.tool_choice.value
        elif isinstance(self.tool_choice, ChatCompletionAllowedToolChoiceParam):
            tool_choice = self.tool_choice.to_dict()
        elif isinstance(self.tool_choice, ChatCompletionNamedToolChoiceParam):
            tool_choice = self.tool_choice.to_dict()
        elif isinstance(self.tool_choice, ChatCompletionNamedToolChoiceCustomParam):
            tool_choice = self.tool_choice.to_dict()
        else:
            tool_choice = self.tool_choice

        tools: list[dict[str, Any] | str] | None | Unset
        if isinstance(self.tools, Unset):
            tools = UNSET
        elif isinstance(self.tools, list):
            tools = []
            for tools_type_0_item_data in self.tools:
                tools_type_0_item: dict[str, Any] | str
                if isinstance(tools_type_0_item_data, ChatCompletionFunctionToolParam):
                    tools_type_0_item = tools_type_0_item_data.to_dict()
                else:
                    tools_type_0_item = tools_type_0_item_data
                tools.append(tools_type_0_item)

        else:
            tools = self.tools

        top_logprobs: int | None | Unset
        if isinstance(self.top_logprobs, Unset):
            top_logprobs = UNSET
        else:
            top_logprobs = self.top_logprobs

        top_p: float | None | Unset
        if isinstance(self.top_p, Unset):
            top_p = UNSET
        else:
            top_p = self.top_p

        user: None | str | Unset
        if isinstance(self.user, Unset):
            user = UNSET
        else:
            user = self.user

        verbosity: None | str | Unset
        if isinstance(self.verbosity, Unset):
            verbosity = UNSET
        elif isinstance(self.verbosity, LlmChatOpenaiConfigVerbosityType0):
            verbosity = self.verbosity.value
        else:
            verbosity = self.verbosity

        web_search_options: dict[str, Any] | None | Unset
        if isinstance(self.web_search_options, Unset):
            web_search_options = UNSET
        elif isinstance(self.web_search_options, WebSearchOptions):
            web_search_options = self.web_search_options.to_dict()
        else:
            web_search_options = self.web_search_options

        output_type: None | str | Unset
        if isinstance(self.output_type, Unset):
            output_type = UNSET
        elif isinstance(self.output_type, OutputType):
            output_type = self.output_type.value
        else:
            output_type = self.output_type

        json_schema: dict[str, Any] | None | Unset
        if isinstance(self.json_schema, Unset):
            json_schema = UNSET
        elif isinstance(self.json_schema, LlmChatOpenaiConfigJsonSchemaType0):
            json_schema = self.json_schema.to_dict()
        else:
            json_schema = self.json_schema

        response_schema: dict[str, Any] | None | Unset
        if isinstance(self.response_schema, Unset):
            response_schema = UNSET
        elif isinstance(self.response_schema, StructuredOutputSchema):
            response_schema = self.response_schema.to_dict()
        elif isinstance(self.response_schema, LlmChatOpenaiConfigResponseSchemaType1):
            response_schema = self.response_schema.to_dict()
        else:
            response_schema = self.response_schema

        extra_headers: dict[str, Any] | None | Unset
        if isinstance(self.extra_headers, Unset):
            extra_headers = UNSET
        elif isinstance(self.extra_headers, LlmChatOpenaiConfigExtraHeadersType0):
            extra_headers = self.extra_headers.to_dict()
        else:
            extra_headers = self.extra_headers

        extra_query: dict[str, Any] | None | Unset
        if isinstance(self.extra_query, Unset):
            extra_query = UNSET
        elif isinstance(self.extra_query, LlmChatOpenaiConfigExtraQueryType0):
            extra_query = self.extra_query.to_dict()
        else:
            extra_query = self.extra_query

        extra_body: Any | None | Unset
        if isinstance(self.extra_body, Unset):
            extra_body = UNSET
        else:
            extra_body = self.extra_body

        timeout: float | None | Unset
        if isinstance(self.timeout, Unset):
            timeout = UNSET
        else:
            timeout = self.timeout

        tool_output_text_to_file_char_limit: int | None | Unset
        if isinstance(self.tool_output_text_to_file_char_limit, Unset):
            tool_output_text_to_file_char_limit = UNSET
        else:
            tool_output_text_to_file_char_limit = self.tool_output_text_to_file_char_limit

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "messages": messages,
                "provider": provider,
            }
        )
        if model is not UNSET:
            field_dict["model"] = model
        if audio is not UNSET:
            field_dict["audio"] = audio
        if frequency_penalty is not UNSET:
            field_dict["frequency_penalty"] = frequency_penalty
        if logit_bias is not UNSET:
            field_dict["logit_bias"] = logit_bias
        if logprobs is not UNSET:
            field_dict["logprobs"] = logprobs
        if max_completion_tokens is not UNSET:
            field_dict["max_completion_tokens"] = max_completion_tokens
        if max_tokens is not UNSET:
            field_dict["max_tokens"] = max_tokens
        if metadata is not UNSET:
            field_dict["metadata"] = metadata
        if modalities is not UNSET:
            field_dict["modalities"] = modalities
        if n is not UNSET:
            field_dict["n"] = n
        if parallel_tool_calls is not UNSET:
            field_dict["parallel_tool_calls"] = parallel_tool_calls
        if prediction is not UNSET:
            field_dict["prediction"] = prediction
        if presence_penalty is not UNSET:
            field_dict["presence_penalty"] = presence_penalty
        if prompt_cache_key is not UNSET:
            field_dict["prompt_cache_key"] = prompt_cache_key
        if prompt_cache_retention is not UNSET:
            field_dict["prompt_cache_retention"] = prompt_cache_retention
        if reasoning_effort is not UNSET:
            field_dict["reasoning_effort"] = reasoning_effort
        if response_format is not UNSET:
            field_dict["response_format"] = response_format
        if safety_identifier is not UNSET:
            field_dict["safety_identifier"] = safety_identifier
        if seed is not UNSET:
            field_dict["seed"] = seed
        if service_tier is not UNSET:
            field_dict["service_tier"] = service_tier
        if stop is not UNSET:
            field_dict["stop"] = stop
        if store is not UNSET:
            field_dict["store"] = store
        if stream is not UNSET:
            field_dict["stream"] = stream
        if stream_options is not UNSET:
            field_dict["stream_options"] = stream_options
        if temperature is not UNSET:
            field_dict["temperature"] = temperature
        if tool_choice is not UNSET:
            field_dict["tool_choice"] = tool_choice
        if tools is not UNSET:
            field_dict["tools"] = tools
        if top_logprobs is not UNSET:
            field_dict["top_logprobs"] = top_logprobs
        if top_p is not UNSET:
            field_dict["top_p"] = top_p
        if user is not UNSET:
            field_dict["user"] = user
        if verbosity is not UNSET:
            field_dict["verbosity"] = verbosity
        if web_search_options is not UNSET:
            field_dict["web_search_options"] = web_search_options
        if output_type is not UNSET:
            field_dict["output_type"] = output_type
        if json_schema is not UNSET:
            field_dict["json_schema"] = json_schema
        if response_schema is not UNSET:
            field_dict["response_schema"] = response_schema
        if extra_headers is not UNSET:
            field_dict["extra_headers"] = extra_headers
        if extra_query is not UNSET:
            field_dict["extra_query"] = extra_query
        if extra_body is not UNSET:
            field_dict["extra_body"] = extra_body
        if timeout is not UNSET:
            field_dict["timeout"] = timeout
        if tool_output_text_to_file_char_limit is not UNSET:
            field_dict["tool_output_text_to_file_char_limit"] = tool_output_text_to_file_char_limit

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.chat_completion_allowed_tool_choice_param import ChatCompletionAllowedToolChoiceParam
        from ..models.chat_completion_assistant_message_param import ChatCompletionAssistantMessageParam
        from ..models.chat_completion_audio_param import ChatCompletionAudioParam
        from ..models.chat_completion_developer_message_param import ChatCompletionDeveloperMessageParam
        from ..models.chat_completion_function_message_param import ChatCompletionFunctionMessageParam
        from ..models.chat_completion_function_tool_param import ChatCompletionFunctionToolParam
        from ..models.chat_completion_named_tool_choice_custom_param import ChatCompletionNamedToolChoiceCustomParam
        from ..models.chat_completion_named_tool_choice_param import ChatCompletionNamedToolChoiceParam
        from ..models.chat_completion_prediction_content_param import ChatCompletionPredictionContentParam
        from ..models.chat_completion_stream_options_param import ChatCompletionStreamOptionsParam
        from ..models.chat_completion_system_message_param import ChatCompletionSystemMessageParam
        from ..models.chat_completion_tool_message_param import ChatCompletionToolMessageParam
        from ..models.chat_completion_user_message_param import ChatCompletionUserMessageParam
        from ..models.llm_chat_openai_config_extra_headers_type_0 import LlmChatOpenaiConfigExtraHeadersType0
        from ..models.llm_chat_openai_config_extra_query_type_0 import LlmChatOpenaiConfigExtraQueryType0
        from ..models.llm_chat_openai_config_json_schema_type_0 import LlmChatOpenaiConfigJsonSchemaType0
        from ..models.llm_chat_openai_config_logit_bias_type_0 import LlmChatOpenaiConfigLogitBiasType0
        from ..models.llm_chat_openai_config_metadata_type_0 import LlmChatOpenaiConfigMetadataType0
        from ..models.llm_chat_openai_config_response_schema_type_1 import LlmChatOpenaiConfigResponseSchemaType1
        from ..models.response_format_json_object import ResponseFormatJSONObject
        from ..models.response_format_json_schema import ResponseFormatJSONSchema
        from ..models.response_format_text import ResponseFormatText
        from ..models.structured_output_schema import StructuredOutputSchema
        from ..models.web_search_options import WebSearchOptions

        d = dict(src_dict)
        messages = []
        _messages = d.pop("messages")
        for messages_item_data in _messages:

            def _parse_messages_item(
                data: object,
            ) -> (
                ChatCompletionAssistantMessageParam
                | ChatCompletionDeveloperMessageParam
                | ChatCompletionFunctionMessageParam
                | ChatCompletionSystemMessageParam
                | ChatCompletionToolMessageParam
                | ChatCompletionUserMessageParam
            ):
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    messages_item_type_0 = ChatCompletionDeveloperMessageParam.from_dict(data)

                    return messages_item_type_0
                except (TypeError, ValueError, AttributeError, KeyError):
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    messages_item_type_1 = ChatCompletionSystemMessageParam.from_dict(data)

                    return messages_item_type_1
                except (TypeError, ValueError, AttributeError, KeyError):
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    messages_item_type_2 = ChatCompletionUserMessageParam.from_dict(data)

                    return messages_item_type_2
                except (TypeError, ValueError, AttributeError, KeyError):
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    messages_item_type_3 = ChatCompletionAssistantMessageParam.from_dict(data)

                    return messages_item_type_3
                except (TypeError, ValueError, AttributeError, KeyError):
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    messages_item_type_4 = ChatCompletionToolMessageParam.from_dict(data)

                    return messages_item_type_4
                except (TypeError, ValueError, AttributeError, KeyError):
                    pass
                if not isinstance(data, dict):
                    raise TypeError()
                messages_item_type_5 = ChatCompletionFunctionMessageParam.from_dict(data)

                return messages_item_type_5

            messages_item = _parse_messages_item(messages_item_data)

            messages.append(messages_item)

        provider = LlmChatOpenaiConfigProvider(d.pop("provider"))

        def _parse_model(data: object) -> LlmChatOpenaiConfigModelType0 | LlmChatOpenaiConfigModelType1 | str | Unset:
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                model_type_0 = LlmChatOpenaiConfigModelType0(data)

                return model_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, str):
                    raise TypeError()
                model_type_1 = LlmChatOpenaiConfigModelType1(data)

                return model_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(LlmChatOpenaiConfigModelType0 | LlmChatOpenaiConfigModelType1 | str | Unset, data)

        model = _parse_model(d.pop("model", UNSET))

        def _parse_audio(data: object) -> ChatCompletionAudioParam | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                audio_type_0 = ChatCompletionAudioParam.from_dict(data)

                return audio_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(ChatCompletionAudioParam | None | Unset, data)

        audio = _parse_audio(d.pop("audio", UNSET))

        def _parse_frequency_penalty(data: object) -> float | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(float | None | Unset, data)

        frequency_penalty = _parse_frequency_penalty(d.pop("frequency_penalty", UNSET))

        def _parse_logit_bias(data: object) -> LlmChatOpenaiConfigLogitBiasType0 | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                logit_bias_type_0 = LlmChatOpenaiConfigLogitBiasType0.from_dict(data)

                return logit_bias_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(LlmChatOpenaiConfigLogitBiasType0 | None | Unset, data)

        logit_bias = _parse_logit_bias(d.pop("logit_bias", UNSET))

        def _parse_logprobs(data: object) -> bool | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(bool | None | Unset, data)

        logprobs = _parse_logprobs(d.pop("logprobs", UNSET))

        def _parse_max_completion_tokens(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        max_completion_tokens = _parse_max_completion_tokens(d.pop("max_completion_tokens", UNSET))

        def _parse_max_tokens(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        max_tokens = _parse_max_tokens(d.pop("max_tokens", UNSET))

        def _parse_metadata(data: object) -> LlmChatOpenaiConfigMetadataType0 | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                metadata_type_0 = LlmChatOpenaiConfigMetadataType0.from_dict(data)

                return metadata_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(LlmChatOpenaiConfigMetadataType0 | None | Unset, data)

        metadata = _parse_metadata(d.pop("metadata", UNSET))

        def _parse_modalities(data: object) -> list[LlmChatOpenaiConfigModalitiesType0Item] | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                modalities_type_0 = []
                _modalities_type_0 = data
                for modalities_type_0_item_data in _modalities_type_0:
                    modalities_type_0_item = LlmChatOpenaiConfigModalitiesType0Item(modalities_type_0_item_data)

                    modalities_type_0.append(modalities_type_0_item)

                return modalities_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[LlmChatOpenaiConfigModalitiesType0Item] | None | Unset, data)

        modalities = _parse_modalities(d.pop("modalities", UNSET))

        def _parse_n(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        n = _parse_n(d.pop("n", UNSET))

        def _parse_parallel_tool_calls(data: object) -> bool | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(bool | None | Unset, data)

        parallel_tool_calls = _parse_parallel_tool_calls(d.pop("parallel_tool_calls", UNSET))

        def _parse_prediction(data: object) -> ChatCompletionPredictionContentParam | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                prediction_type_0 = ChatCompletionPredictionContentParam.from_dict(data)

                return prediction_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(ChatCompletionPredictionContentParam | None | Unset, data)

        prediction = _parse_prediction(d.pop("prediction", UNSET))

        def _parse_presence_penalty(data: object) -> float | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(float | None | Unset, data)

        presence_penalty = _parse_presence_penalty(d.pop("presence_penalty", UNSET))

        def _parse_prompt_cache_key(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        prompt_cache_key = _parse_prompt_cache_key(d.pop("prompt_cache_key", UNSET))

        def _parse_prompt_cache_retention(data: object) -> LlmChatOpenaiConfigPromptCacheRetentionType0 | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                prompt_cache_retention_type_0 = LlmChatOpenaiConfigPromptCacheRetentionType0(data)

                return prompt_cache_retention_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(LlmChatOpenaiConfigPromptCacheRetentionType0 | None | Unset, data)

        prompt_cache_retention = _parse_prompt_cache_retention(d.pop("prompt_cache_retention", UNSET))

        def _parse_reasoning_effort(data: object) -> LlmChatOpenaiConfigReasoningEffortType0 | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                reasoning_effort_type_0 = LlmChatOpenaiConfigReasoningEffortType0(data)

                return reasoning_effort_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(LlmChatOpenaiConfigReasoningEffortType0 | None | Unset, data)

        reasoning_effort = _parse_reasoning_effort(d.pop("reasoning_effort", UNSET))

        def _parse_response_format(
            data: object,
        ) -> None | ResponseFormatJSONObject | ResponseFormatJSONSchema | ResponseFormatText | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_format_type_0 = ResponseFormatText.from_dict(data)

                return response_format_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_format_type_1 = ResponseFormatJSONSchema.from_dict(data)

                return response_format_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_format_type_2 = ResponseFormatJSONObject.from_dict(data)

                return response_format_type_2
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | ResponseFormatJSONObject | ResponseFormatJSONSchema | ResponseFormatText | Unset, data)

        response_format = _parse_response_format(d.pop("response_format", UNSET))

        def _parse_safety_identifier(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        safety_identifier = _parse_safety_identifier(d.pop("safety_identifier", UNSET))

        def _parse_seed(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        seed = _parse_seed(d.pop("seed", UNSET))

        def _parse_service_tier(data: object) -> LlmChatOpenaiConfigServiceTierType0 | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                service_tier_type_0 = LlmChatOpenaiConfigServiceTierType0(data)

                return service_tier_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(LlmChatOpenaiConfigServiceTierType0 | None | Unset, data)

        service_tier = _parse_service_tier(d.pop("service_tier", UNSET))

        def _parse_stop(data: object) -> list[str] | None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                stop_type_1 = cast(list[str], data)

                return stop_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[str] | None | str | Unset, data)

        stop = _parse_stop(d.pop("stop", UNSET))

        def _parse_store(data: object) -> bool | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(bool | None | Unset, data)

        store = _parse_store(d.pop("store", UNSET))

        def _parse_stream(data: object) -> bool | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(bool | None | Unset, data)

        stream = _parse_stream(d.pop("stream", UNSET))

        def _parse_stream_options(data: object) -> ChatCompletionStreamOptionsParam | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                stream_options_type_0 = ChatCompletionStreamOptionsParam.from_dict(data)

                return stream_options_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(ChatCompletionStreamOptionsParam | None | Unset, data)

        stream_options = _parse_stream_options(d.pop("stream_options", UNSET))

        def _parse_temperature(data: object) -> float | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(float | None | Unset, data)

        temperature = _parse_temperature(d.pop("temperature", UNSET))

        def _parse_tool_choice(
            data: object,
        ) -> (
            ChatCompletionAllowedToolChoiceParam
            | ChatCompletionNamedToolChoiceCustomParam
            | ChatCompletionNamedToolChoiceParam
            | LlmChatOpenaiConfigToolChoiceType0
            | None
            | Unset
        ):
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                tool_choice_type_0 = LlmChatOpenaiConfigToolChoiceType0(data)

                return tool_choice_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                tool_choice_type_1 = ChatCompletionAllowedToolChoiceParam.from_dict(data)

                return tool_choice_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                tool_choice_type_2 = ChatCompletionNamedToolChoiceParam.from_dict(data)

                return tool_choice_type_2
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                tool_choice_type_3 = ChatCompletionNamedToolChoiceCustomParam.from_dict(data)

                return tool_choice_type_3
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(
                ChatCompletionAllowedToolChoiceParam
                | ChatCompletionNamedToolChoiceCustomParam
                | ChatCompletionNamedToolChoiceParam
                | LlmChatOpenaiConfigToolChoiceType0
                | None
                | Unset,
                data,
            )

        tool_choice = _parse_tool_choice(d.pop("tool_choice", UNSET))

        def _parse_tools(data: object) -> list[ChatCompletionFunctionToolParam | str] | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                tools_type_0 = []
                _tools_type_0 = data
                for tools_type_0_item_data in _tools_type_0:

                    def _parse_tools_type_0_item(data: object) -> ChatCompletionFunctionToolParam | str:
                        try:
                            if not isinstance(data, dict):
                                raise TypeError()
                            tools_type_0_item_type_0 = ChatCompletionFunctionToolParam.from_dict(data)

                            return tools_type_0_item_type_0
                        except (TypeError, ValueError, AttributeError, KeyError):
                            pass
                        return cast(ChatCompletionFunctionToolParam | str, data)

                    tools_type_0_item = _parse_tools_type_0_item(tools_type_0_item_data)

                    tools_type_0.append(tools_type_0_item)

                return tools_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[ChatCompletionFunctionToolParam | str] | None | Unset, data)

        tools = _parse_tools(d.pop("tools", UNSET))

        def _parse_top_logprobs(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        top_logprobs = _parse_top_logprobs(d.pop("top_logprobs", UNSET))

        def _parse_top_p(data: object) -> float | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(float | None | Unset, data)

        top_p = _parse_top_p(d.pop("top_p", UNSET))

        def _parse_user(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        user = _parse_user(d.pop("user", UNSET))

        def _parse_verbosity(data: object) -> LlmChatOpenaiConfigVerbosityType0 | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                verbosity_type_0 = LlmChatOpenaiConfigVerbosityType0(data)

                return verbosity_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(LlmChatOpenaiConfigVerbosityType0 | None | Unset, data)

        verbosity = _parse_verbosity(d.pop("verbosity", UNSET))

        def _parse_web_search_options(data: object) -> None | Unset | WebSearchOptions:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                web_search_options_type_0 = WebSearchOptions.from_dict(data)

                return web_search_options_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Unset | WebSearchOptions, data)

        web_search_options = _parse_web_search_options(d.pop("web_search_options", UNSET))

        def _parse_output_type(data: object) -> None | OutputType | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                output_type_type_0 = OutputType(data)

                return output_type_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | OutputType | Unset, data)

        output_type = _parse_output_type(d.pop("output_type", UNSET))

        def _parse_json_schema(data: object) -> LlmChatOpenaiConfigJsonSchemaType0 | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                json_schema_type_0 = LlmChatOpenaiConfigJsonSchemaType0.from_dict(data)

                return json_schema_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(LlmChatOpenaiConfigJsonSchemaType0 | None | Unset, data)

        json_schema = _parse_json_schema(d.pop("json_schema", UNSET))

        def _parse_response_schema(
            data: object,
        ) -> LlmChatOpenaiConfigResponseSchemaType1 | None | StructuredOutputSchema | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_schema_type_0 = StructuredOutputSchema.from_dict(data)

                return response_schema_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_schema_type_1 = LlmChatOpenaiConfigResponseSchemaType1.from_dict(data)

                return response_schema_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(LlmChatOpenaiConfigResponseSchemaType1 | None | StructuredOutputSchema | Unset, data)

        response_schema = _parse_response_schema(d.pop("response_schema", UNSET))

        def _parse_extra_headers(data: object) -> LlmChatOpenaiConfigExtraHeadersType0 | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                extra_headers_type_0 = LlmChatOpenaiConfigExtraHeadersType0.from_dict(data)

                return extra_headers_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(LlmChatOpenaiConfigExtraHeadersType0 | None | Unset, data)

        extra_headers = _parse_extra_headers(d.pop("extra_headers", UNSET))

        def _parse_extra_query(data: object) -> LlmChatOpenaiConfigExtraQueryType0 | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                extra_query_type_0 = LlmChatOpenaiConfigExtraQueryType0.from_dict(data)

                return extra_query_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(LlmChatOpenaiConfigExtraQueryType0 | None | Unset, data)

        extra_query = _parse_extra_query(d.pop("extra_query", UNSET))

        def _parse_extra_body(data: object) -> Any | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Any | None | Unset, data)

        extra_body = _parse_extra_body(d.pop("extra_body", UNSET))

        def _parse_timeout(data: object) -> float | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(float | None | Unset, data)

        timeout = _parse_timeout(d.pop("timeout", UNSET))

        def _parse_tool_output_text_to_file_char_limit(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        tool_output_text_to_file_char_limit = _parse_tool_output_text_to_file_char_limit(
            d.pop("tool_output_text_to_file_char_limit", UNSET)
        )

        llm_chat_openai_config = cls(
            messages=messages,
            provider=provider,
            model=model,
            audio=audio,
            frequency_penalty=frequency_penalty,
            logit_bias=logit_bias,
            logprobs=logprobs,
            max_completion_tokens=max_completion_tokens,
            max_tokens=max_tokens,
            metadata=metadata,
            modalities=modalities,
            n=n,
            parallel_tool_calls=parallel_tool_calls,
            prediction=prediction,
            presence_penalty=presence_penalty,
            prompt_cache_key=prompt_cache_key,
            prompt_cache_retention=prompt_cache_retention,
            reasoning_effort=reasoning_effort,
            response_format=response_format,
            safety_identifier=safety_identifier,
            seed=seed,
            service_tier=service_tier,
            stop=stop,
            store=store,
            stream=stream,
            stream_options=stream_options,
            temperature=temperature,
            tool_choice=tool_choice,
            tools=tools,
            top_logprobs=top_logprobs,
            top_p=top_p,
            user=user,
            verbosity=verbosity,
            web_search_options=web_search_options,
            output_type=output_type,
            json_schema=json_schema,
            response_schema=response_schema,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
            tool_output_text_to_file_char_limit=tool_output_text_to_file_char_limit,
        )

        llm_chat_openai_config.additional_properties = d
        return llm_chat_openai_config

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, Literal, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.vector_store_file_batch_status import VectorStoreFileBatchStatus

if TYPE_CHECKING:
    from ..models.file_counts import FileCounts


T = TypeVar("T", bound="VectorStoreFileBatch")


@_attrs_define
class VectorStoreFileBatch:
    """A batch of files attached to a vector store.

    Attributes:
        id (str):
        created_at (int):
        file_counts (FileCounts):
        object_ (Literal['vector_store.files_batch']):
        status (VectorStoreFileBatchStatus):
        vector_store_id (str):
    """

    id: str
    created_at: int
    file_counts: FileCounts
    object_: Literal["vector_store.files_batch"]
    status: VectorStoreFileBatchStatus
    vector_store_id: str
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        created_at = self.created_at

        file_counts = self.file_counts.to_dict()

        object_ = self.object_

        status = self.status.value

        vector_store_id = self.vector_store_id

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "created_at": created_at,
                "file_counts": file_counts,
                "object": object_,
                "status": status,
                "vector_store_id": vector_store_id,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.file_counts import FileCounts

        d = dict(src_dict)
        id = d.pop("id")

        created_at = d.pop("created_at")

        file_counts = FileCounts.from_dict(d.pop("file_counts"))

        object_ = cast(Literal["vector_store.files_batch"], d.pop("object"))
        if object_ != "vector_store.files_batch":
            raise ValueError(f"object must match const 'vector_store.files_batch', got '{object_}'")

        status = VectorStoreFileBatchStatus(d.pop("status"))

        vector_store_id = d.pop("vector_store_id")

        vector_store_file_batch = cls(
            id=id,
            created_at=created_at,
            file_counts=file_counts,
            object_=object_,
            status=status,
            vector_store_id=vector_store_id,
        )

        vector_store_file_batch.additional_properties = d
        return vector_store_file_batch

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.http_validation_error import HTTPValidationError
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    code: None | str | Unset = UNSET,
    state: None | str | Unset = UNSET,
    error: None | str | Unset = UNSET,
) -> dict[str, Any]:

    params: dict[str, Any] = {}

    json_code: None | str | Unset
    if isinstance(code, Unset):
        json_code = UNSET
    else:
        json_code = code
    params["code"] = json_code

    json_state: None | str | Unset
    if isinstance(state, Unset):
        json_state = UNSET
    else:
        json_state = state
    params["state"] = json_state

    json_error: None | str | Unset
    if isinstance(error, Unset):
        json_error = UNSET
    else:
        json_error = error
    params["error"] = json_error

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/v1/auth/ukg/oauth-callback",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Any | HTTPValidationError | None:
    if response.status_code == 200:
        response_200 = response.json()
        return response_200

    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Response[Any | HTTPValidationError]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    code: None | str | Unset = UNSET,
    state: None | str | Unset = UNSET,
    error: None | str | Unset = UNSET,
) -> Response[Any | HTTPValidationError]:
    """Ukg Oauth Callback

     Handle UKG OAuth callback after user authorization.

    This endpoint receives the authorization code from UKG and exchanges it
    for an access token, then stores it in the user's app_auth.

    Args:
        code (None | str | Unset):
        state (None | str | Unset):
        error (None | str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any | HTTPValidationError]
    """

    kwargs = _get_kwargs(
        code=code,
        state=state,
        error=error,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    code: None | str | Unset = UNSET,
    state: None | str | Unset = UNSET,
    error: None | str | Unset = UNSET,
) -> Any | HTTPValidationError | None:
    """Ukg Oauth Callback

     Handle UKG OAuth callback after user authorization.

    This endpoint receives the authorization code from UKG and exchanges it
    for an access token, then stores it in the user's app_auth.

    Args:
        code (None | str | Unset):
        state (None | str | Unset):
        error (None | str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Any | HTTPValidationError
    """

    return sync_detailed(
        client=client,
        code=code,
        state=state,
        error=error,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    code: None | str | Unset = UNSET,
    state: None | str | Unset = UNSET,
    error: None | str | Unset = UNSET,
) -> Response[Any | HTTPValidationError]:
    """Ukg Oauth Callback

     Handle UKG OAuth callback after user authorization.

    This endpoint receives the authorization code from UKG and exchanges it
    for an access token, then stores it in the user's app_auth.

    Args:
        code (None | str | Unset):
        state (None | str | Unset):
        error (None | str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any | HTTPValidationError]
    """

    kwargs = _get_kwargs(
        code=code,
        state=state,
        error=error,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    code: None | str | Unset = UNSET,
    state: None | str | Unset = UNSET,
    error: None | str | Unset = UNSET,
) -> Any | HTTPValidationError | None:
    """Ukg Oauth Callback

     Handle UKG OAuth callback after user authorization.

    This endpoint receives the authorization code from UKG and exchanges it
    for an access token, then stores it in the user's app_auth.

    Args:
        code (None | str | Unset):
        state (None | str | Unset):
        error (None | str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Any | HTTPValidationError
    """

    return (
        await asyncio_detailed(
            client=client,
            code=code,
            state=state,
            error=error,
        )
    ).parsed



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, Literal, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.dalle_openai_config_provider import DalleOpenaiConfigProvider
from ..models.dalle_openai_config_quality import DalleOpenaiConfigQuality
from ..models.dalle_openai_config_response_format_type_0 import DalleOpenaiConfigResponseFormatType0
from ..models.dalle_openai_config_size_type_0 import DalleOpenaiConfigSizeType0
from ..models.dalle_openai_config_style_type_0 import DalleOpenaiConfigStyleType0
from ..types import UNSET, Unset

T = TypeVar("T", bound="DalleOpenaiConfig")


@_attrs_define
class DalleOpenaiConfig:
    """
    Attributes:
        prompt (str):
        provider (DalleOpenaiConfigProvider):
        model (Literal['dall-e-3'] | Unset):  Default: 'dall-e-3'.
        n (int | None | Unset):  Default: 1.
        quality (DalleOpenaiConfigQuality | Unset):  Default: DalleOpenaiConfigQuality.STANDARD.
        response_format (DalleOpenaiConfigResponseFormatType0 | None | Unset):  Default:
            DalleOpenaiConfigResponseFormatType0.URL.
        size (DalleOpenaiConfigSizeType0 | None | Unset):  Default: DalleOpenaiConfigSizeType0.VALUE_2.
        style (DalleOpenaiConfigStyleType0 | None | Unset):  Default: DalleOpenaiConfigStyleType0.NATURAL.
        user (str | Unset):  Default: ''.
    """

    prompt: str
    provider: DalleOpenaiConfigProvider
    model: Literal["dall-e-3"] | Unset = "dall-e-3"
    n: int | None | Unset = 1
    quality: DalleOpenaiConfigQuality | Unset = DalleOpenaiConfigQuality.STANDARD
    response_format: DalleOpenaiConfigResponseFormatType0 | None | Unset = DalleOpenaiConfigResponseFormatType0.URL
    size: DalleOpenaiConfigSizeType0 | None | Unset = DalleOpenaiConfigSizeType0.VALUE_2
    style: DalleOpenaiConfigStyleType0 | None | Unset = DalleOpenaiConfigStyleType0.NATURAL
    user: str | Unset = ""
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        prompt = self.prompt

        provider = self.provider.value

        model = self.model

        n: int | None | Unset
        if isinstance(self.n, Unset):
            n = UNSET
        else:
            n = self.n

        quality: str | Unset = UNSET
        if not isinstance(self.quality, Unset):
            quality = self.quality.value

        response_format: None | str | Unset
        if isinstance(self.response_format, Unset):
            response_format = UNSET
        elif isinstance(self.response_format, DalleOpenaiConfigResponseFormatType0):
            response_format = self.response_format.value
        else:
            response_format = self.response_format

        size: None | str | Unset
        if isinstance(self.size, Unset):
            size = UNSET
        elif isinstance(self.size, DalleOpenaiConfigSizeType0):
            size = self.size.value
        else:
            size = self.size

        style: None | str | Unset
        if isinstance(self.style, Unset):
            style = UNSET
        elif isinstance(self.style, DalleOpenaiConfigStyleType0):
            style = self.style.value
        else:
            style = self.style

        user = self.user

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "prompt": prompt,
                "provider": provider,
            }
        )
        if model is not UNSET:
            field_dict["model"] = model
        if n is not UNSET:
            field_dict["n"] = n
        if quality is not UNSET:
            field_dict["quality"] = quality
        if response_format is not UNSET:
            field_dict["response_format"] = response_format
        if size is not UNSET:
            field_dict["size"] = size
        if style is not UNSET:
            field_dict["style"] = style
        if user is not UNSET:
            field_dict["user"] = user

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        prompt = d.pop("prompt")

        provider = DalleOpenaiConfigProvider(d.pop("provider"))

        model = cast(Literal["dall-e-3"] | Unset, d.pop("model", UNSET))
        if model != "dall-e-3" and not isinstance(model, Unset):
            raise ValueError(f"model must match const 'dall-e-3', got '{model}'")

        def _parse_n(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        n = _parse_n(d.pop("n", UNSET))

        _quality = d.pop("quality", UNSET)
        quality: DalleOpenaiConfigQuality | Unset
        if isinstance(_quality, Unset):
            quality = UNSET
        else:
            quality = DalleOpenaiConfigQuality(_quality)

        def _parse_response_format(data: object) -> DalleOpenaiConfigResponseFormatType0 | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                response_format_type_0 = DalleOpenaiConfigResponseFormatType0(data)

                return response_format_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(DalleOpenaiConfigResponseFormatType0 | None | Unset, data)

        response_format = _parse_response_format(d.pop("response_format", UNSET))

        def _parse_size(data: object) -> DalleOpenaiConfigSizeType0 | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                size_type_0 = DalleOpenaiConfigSizeType0(data)

                return size_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(DalleOpenaiConfigSizeType0 | None | Unset, data)

        size = _parse_size(d.pop("size", UNSET))

        def _parse_style(data: object) -> DalleOpenaiConfigStyleType0 | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                style_type_0 = DalleOpenaiConfigStyleType0(data)

                return style_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(DalleOpenaiConfigStyleType0 | None | Unset, data)

        style = _parse_style(d.pop("style", UNSET))

        user = d.pop("user", UNSET)

        dalle_openai_config = cls(
            prompt=prompt,
            provider=provider,
            model=model,
            n=n,
            quality=quality,
            response_format=response_format,
            size=size,
            style=style,
            user=user,
        )

        dalle_openai_config.additional_properties = d
        return dalle_openai_config

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ListActionToolsV1ActionsToolsGetLlmType(str, Enum):
    CLAUDE = "claude"
    GEMINI = "gemini"
    GROK = "grok"
    OPENAI_ASSISTANT = "openai_assistant"
    OPENAI_CHAT = "openai_chat"
    OPENAI_RESPONSES = "openai_responses"
    VALUE_0 = ""

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="UsageInputTokensDetails")


@_attrs_define
class UsageInputTokensDetails:
    """The input tokens detailed information for the image generation.

    Attributes:
        image_tokens (int):
        text_tokens (int):
    """

    image_tokens: int
    text_tokens: int
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        image_tokens = self.image_tokens

        text_tokens = self.text_tokens

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "image_tokens": image_tokens,
                "text_tokens": text_tokens,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        image_tokens = d.pop("image_tokens")

        text_tokens = d.pop("text_tokens")

        usage_input_tokens_details = cls(
            image_tokens=image_tokens,
            text_tokens=text_tokens,
        )

        usage_input_tokens_details.additional_properties = d
        return usage_input_tokens_details

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class TimeRange(str, Enum):
    CUSTOM = "custom"
    LAST_30_DAYS = "last_30_days"
    LAST_7_DAYS = "last_7_days"
    LAST_90_DAYS = "last_90_days"
    LAST_YEAR = "last_year"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.factor import Factor
    from ..models.user_app_metadata import UserAppMetadata
    from ..models.user_identity import UserIdentity
    from ..models.user_user_metadata import UserUserMetadata


T = TypeVar("T", bound="User")


@_attrs_define
class User:
    """
    Attributes:
        id (str):
        app_metadata (UserAppMetadata):
        user_metadata (UserUserMetadata):
        aud (str):
        created_at (datetime.datetime):
        confirmation_sent_at (datetime.datetime | None | Unset):
        recovery_sent_at (datetime.datetime | None | Unset):
        email_change_sent_at (datetime.datetime | None | Unset):
        new_email (None | str | Unset):
        new_phone (None | str | Unset):
        invited_at (datetime.datetime | None | Unset):
        action_link (None | str | Unset):
        email (None | str | Unset):
        phone (None | str | Unset):
        confirmed_at (datetime.datetime | None | Unset):
        email_confirmed_at (datetime.datetime | None | Unset):
        phone_confirmed_at (datetime.datetime | None | Unset):
        last_sign_in_at (datetime.datetime | None | Unset):
        role (None | str | Unset):
        updated_at (datetime.datetime | None | Unset):
        identities (list[UserIdentity] | None | Unset):
        is_anonymous (bool | Unset):  Default: False.
        is_sso_user (bool | Unset):  Default: False.
        factors (list[Factor] | None | Unset):
        deleted_at (None | str | Unset):
        banned_until (None | str | Unset):
    """

    id: str
    app_metadata: UserAppMetadata
    user_metadata: UserUserMetadata
    aud: str
    created_at: datetime.datetime
    confirmation_sent_at: datetime.datetime | None | Unset = UNSET
    recovery_sent_at: datetime.datetime | None | Unset = UNSET
    email_change_sent_at: datetime.datetime | None | Unset = UNSET
    new_email: None | str | Unset = UNSET
    new_phone: None | str | Unset = UNSET
    invited_at: datetime.datetime | None | Unset = UNSET
    action_link: None | str | Unset = UNSET
    email: None | str | Unset = UNSET
    phone: None | str | Unset = UNSET
    confirmed_at: datetime.datetime | None | Unset = UNSET
    email_confirmed_at: datetime.datetime | None | Unset = UNSET
    phone_confirmed_at: datetime.datetime | None | Unset = UNSET
    last_sign_in_at: datetime.datetime | None | Unset = UNSET
    role: None | str | Unset = UNSET
    updated_at: datetime.datetime | None | Unset = UNSET
    identities: list[UserIdentity] | None | Unset = UNSET
    is_anonymous: bool | Unset = False
    is_sso_user: bool | Unset = False
    factors: list[Factor] | None | Unset = UNSET
    deleted_at: None | str | Unset = UNSET
    banned_until: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        app_metadata = self.app_metadata.to_dict()

        user_metadata = self.user_metadata.to_dict()

        aud = self.aud

        created_at = self.created_at.isoformat()

        confirmation_sent_at: None | str | Unset
        if isinstance(self.confirmation_sent_at, Unset):
            confirmation_sent_at = UNSET
        elif isinstance(self.confirmation_sent_at, datetime.datetime):
            confirmation_sent_at = self.confirmation_sent_at.isoformat()
        else:
            confirmation_sent_at = self.confirmation_sent_at

        recovery_sent_at: None | str | Unset
        if isinstance(self.recovery_sent_at, Unset):
            recovery_sent_at = UNSET
        elif isinstance(self.recovery_sent_at, datetime.datetime):
            recovery_sent_at = self.recovery_sent_at.isoformat()
        else:
            recovery_sent_at = self.recovery_sent_at

        email_change_sent_at: None | str | Unset
        if isinstance(self.email_change_sent_at, Unset):
            email_change_sent_at = UNSET
        elif isinstance(self.email_change_sent_at, datetime.datetime):
            email_change_sent_at = self.email_change_sent_at.isoformat()
        else:
            email_change_sent_at = self.email_change_sent_at

        new_email: None | str | Unset
        if isinstance(self.new_email, Unset):
            new_email = UNSET
        else:
            new_email = self.new_email

        new_phone: None | str | Unset
        if isinstance(self.new_phone, Unset):
            new_phone = UNSET
        else:
            new_phone = self.new_phone

        invited_at: None | str | Unset
        if isinstance(self.invited_at, Unset):
            invited_at = UNSET
        elif isinstance(self.invited_at, datetime.datetime):
            invited_at = self.invited_at.isoformat()
        else:
            invited_at = self.invited_at

        action_link: None | str | Unset
        if isinstance(self.action_link, Unset):
            action_link = UNSET
        else:
            action_link = self.action_link

        email: None | str | Unset
        if isinstance(self.email, Unset):
            email = UNSET
        else:
            email = self.email

        phone: None | str | Unset
        if isinstance(self.phone, Unset):
            phone = UNSET
        else:
            phone = self.phone

        confirmed_at: None | str | Unset
        if isinstance(self.confirmed_at, Unset):
            confirmed_at = UNSET
        elif isinstance(self.confirmed_at, datetime.datetime):
            confirmed_at = self.confirmed_at.isoformat()
        else:
            confirmed_at = self.confirmed_at

        email_confirmed_at: None | str | Unset
        if isinstance(self.email_confirmed_at, Unset):
            email_confirmed_at = UNSET
        elif isinstance(self.email_confirmed_at, datetime.datetime):
            email_confirmed_at = self.email_confirmed_at.isoformat()
        else:
            email_confirmed_at = self.email_confirmed_at

        phone_confirmed_at: None | str | Unset
        if isinstance(self.phone_confirmed_at, Unset):
            phone_confirmed_at = UNSET
        elif isinstance(self.phone_confirmed_at, datetime.datetime):
            phone_confirmed_at = self.phone_confirmed_at.isoformat()
        else:
            phone_confirmed_at = self.phone_confirmed_at

        last_sign_in_at: None | str | Unset
        if isinstance(self.last_sign_in_at, Unset):
            last_sign_in_at = UNSET
        elif isinstance(self.last_sign_in_at, datetime.datetime):
            last_sign_in_at = self.last_sign_in_at.isoformat()
        else:
            last_sign_in_at = self.last_sign_in_at

        role: None | str | Unset
        if isinstance(self.role, Unset):
            role = UNSET
        else:
            role = self.role

        updated_at: None | str | Unset
        if isinstance(self.updated_at, Unset):
            updated_at = UNSET
        elif isinstance(self.updated_at, datetime.datetime):
            updated_at = self.updated_at.isoformat()
        else:
            updated_at = self.updated_at

        identities: list[dict[str, Any]] | None | Unset
        if isinstance(self.identities, Unset):
            identities = UNSET
        elif isinstance(self.identities, list):
            identities = []
            for identities_type_0_item_data in self.identities:
                identities_type_0_item = identities_type_0_item_data.to_dict()
                identities.append(identities_type_0_item)

        else:
            identities = self.identities

        is_anonymous = self.is_anonymous

        is_sso_user = self.is_sso_user

        factors: list[dict[str, Any]] | None | Unset
        if isinstance(self.factors, Unset):
            factors = UNSET
        elif isinstance(self.factors, list):
            factors = []
            for factors_type_0_item_data in self.factors:
                factors_type_0_item = factors_type_0_item_data.to_dict()
                factors.append(factors_type_0_item)

        else:
            factors = self.factors

        deleted_at: None | str | Unset
        if isinstance(self.deleted_at, Unset):
            deleted_at = UNSET
        else:
            deleted_at = self.deleted_at

        banned_until: None | str | Unset
        if isinstance(self.banned_until, Unset):
            banned_until = UNSET
        else:
            banned_until = self.banned_until

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "app_metadata": app_metadata,
                "user_metadata": user_metadata,
                "aud": aud,
                "created_at": created_at,
            }
        )
        if confirmation_sent_at is not UNSET:
            field_dict["confirmation_sent_at"] = confirmation_sent_at
        if recovery_sent_at is not UNSET:
            field_dict["recovery_sent_at"] = recovery_sent_at
        if email_change_sent_at is not UNSET:
            field_dict["email_change_sent_at"] = email_change_sent_at
        if new_email is not UNSET:
            field_dict["new_email"] = new_email
        if new_phone is not UNSET:
            field_dict["new_phone"] = new_phone
        if invited_at is not UNSET:
            field_dict["invited_at"] = invited_at
        if action_link is not UNSET:
            field_dict["action_link"] = action_link
        if email is not UNSET:
            field_dict["email"] = email
        if phone is not UNSET:
            field_dict["phone"] = phone
        if confirmed_at is not UNSET:
            field_dict["confirmed_at"] = confirmed_at
        if email_confirmed_at is not UNSET:
            field_dict["email_confirmed_at"] = email_confirmed_at
        if phone_confirmed_at is not UNSET:
            field_dict["phone_confirmed_at"] = phone_confirmed_at
        if last_sign_in_at is not UNSET:
            field_dict["last_sign_in_at"] = last_sign_in_at
        if role is not UNSET:
            field_dict["role"] = role
        if updated_at is not UNSET:
            field_dict["updated_at"] = updated_at
        if identities is not UNSET:
            field_dict["identities"] = identities
        if is_anonymous is not UNSET:
            field_dict["is_anonymous"] = is_anonymous
        if is_sso_user is not UNSET:
            field_dict["is_sso_user"] = is_sso_user
        if factors is not UNSET:
            field_dict["factors"] = factors
        if deleted_at is not UNSET:
            field_dict["deleted_at"] = deleted_at
        if banned_until is not UNSET:
            field_dict["banned_until"] = banned_until

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.factor import Factor
        from ..models.user_app_metadata import UserAppMetadata
        from ..models.user_identity import UserIdentity
        from ..models.user_user_metadata import UserUserMetadata

        d = dict(src_dict)
        id = d.pop("id")

        app_metadata = UserAppMetadata.from_dict(d.pop("app_metadata"))

        user_metadata = UserUserMetadata.from_dict(d.pop("user_metadata"))

        aud = d.pop("aud")

        created_at = isoparse(d.pop("created_at"))

        def _parse_confirmation_sent_at(data: object) -> datetime.datetime | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                confirmation_sent_at_type_0 = isoparse(data)

                return confirmation_sent_at_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(datetime.datetime | None | Unset, data)

        confirmation_sent_at = _parse_confirmation_sent_at(d.pop("confirmation_sent_at", UNSET))

        def _parse_recovery_sent_at(data: object) -> datetime.datetime | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                recovery_sent_at_type_0 = isoparse(data)

                return recovery_sent_at_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(datetime.datetime | None | Unset, data)

        recovery_sent_at = _parse_recovery_sent_at(d.pop("recovery_sent_at", UNSET))

        def _parse_email_change_sent_at(data: object) -> datetime.datetime | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                email_change_sent_at_type_0 = isoparse(data)

                return email_change_sent_at_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(datetime.datetime | None | Unset, data)

        email_change_sent_at = _parse_email_change_sent_at(d.pop("email_change_sent_at", UNSET))

        def _parse_new_email(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        new_email = _parse_new_email(d.pop("new_email", UNSET))

        def _parse_new_phone(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        new_phone = _parse_new_phone(d.pop("new_phone", UNSET))

        def _parse_invited_at(data: object) -> datetime.datetime | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                invited_at_type_0 = isoparse(data)

                return invited_at_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(datetime.datetime | None | Unset, data)

        invited_at = _parse_invited_at(d.pop("invited_at", UNSET))

        def _parse_action_link(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        action_link = _parse_action_link(d.pop("action_link", UNSET))

        def _parse_email(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        email = _parse_email(d.pop("email", UNSET))

        def _parse_phone(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        phone = _parse_phone(d.pop("phone", UNSET))

        def _parse_confirmed_at(data: object) -> datetime.datetime | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                confirmed_at_type_0 = isoparse(data)

                return confirmed_at_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(datetime.datetime | None | Unset, data)

        confirmed_at = _parse_confirmed_at(d.pop("confirmed_at", UNSET))

        def _parse_email_confirmed_at(data: object) -> datetime.datetime | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                email_confirmed_at_type_0 = isoparse(data)

                return email_confirmed_at_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(datetime.datetime | None | Unset, data)

        email_confirmed_at = _parse_email_confirmed_at(d.pop("email_confirmed_at", UNSET))

        def _parse_phone_confirmed_at(data: object) -> datetime.datetime | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                phone_confirmed_at_type_0 = isoparse(data)

                return phone_confirmed_at_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(datetime.datetime | None | Unset, data)

        phone_confirmed_at = _parse_phone_confirmed_at(d.pop("phone_confirmed_at", UNSET))

        def _parse_last_sign_in_at(data: object) -> datetime.datetime | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                last_sign_in_at_type_0 = isoparse(data)

                return last_sign_in_at_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(datetime.datetime | None | Unset, data)

        last_sign_in_at = _parse_last_sign_in_at(d.pop("last_sign_in_at", UNSET))

        def _parse_role(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        role = _parse_role(d.pop("role", UNSET))

        def _parse_updated_at(data: object) -> datetime.datetime | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                updated_at_type_0 = isoparse(data)

                return updated_at_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(datetime.datetime | None | Unset, data)

        updated_at = _parse_updated_at(d.pop("updated_at", UNSET))

        def _parse_identities(data: object) -> list[UserIdentity] | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                identities_type_0 = []
                _identities_type_0 = data
                for identities_type_0_item_data in _identities_type_0:
                    identities_type_0_item = UserIdentity.from_dict(identities_type_0_item_data)

                    identities_type_0.append(identities_type_0_item)

                return identities_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[UserIdentity] | None | Unset, data)

        identities = _parse_identities(d.pop("identities", UNSET))

        is_anonymous = d.pop("is_anonymous", UNSET)

        is_sso_user = d.pop("is_sso_user", UNSET)

        def _parse_factors(data: object) -> list[Factor] | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                factors_type_0 = []
                _factors_type_0 = data
                for factors_type_0_item_data in _factors_type_0:
                    factors_type_0_item = Factor.from_dict(factors_type_0_item_data)

                    factors_type_0.append(factors_type_0_item)

                return factors_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[Factor] | None | Unset, data)

        factors = _parse_factors(d.pop("factors", UNSET))

        def _parse_deleted_at(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        deleted_at = _parse_deleted_at(d.pop("deleted_at", UNSET))

        def _parse_banned_until(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        banned_until = _parse_banned_until(d.pop("banned_until", UNSET))

        user = cls(
            id=id,
            app_metadata=app_metadata,
            user_metadata=user_metadata,
            aud=aud,
            created_at=created_at,
            confirmation_sent_at=confirmation_sent_at,
            recovery_sent_at=recovery_sent_at,
            email_change_sent_at=email_change_sent_at,
            new_email=new_email,
            new_phone=new_phone,
            invited_at=invited_at,
            action_link=action_link,
            email=email,
            phone=phone,
            confirmed_at=confirmed_at,
            email_confirmed_at=email_confirmed_at,
            phone_confirmed_at=phone_confirmed_at,
            last_sign_in_at=last_sign_in_at,
            role=role,
            updated_at=updated_at,
            identities=identities,
            is_anonymous=is_anonymous,
            is_sso_user=is_sso_user,
            factors=factors,
            deleted_at=deleted_at,
            banned_until=banned_until,
        )

        user.additional_properties = d
        return user

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class DalleOpenaiConfigSizeType0(str, Enum):
    VALUE_0 = "256x256"
    VALUE_1 = "512x512"
    VALUE_2 = "1024x1024"
    VALUE_3 = "1792x1024"
    VALUE_4 = "1024x1792"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.speech_req_model import SpeechReqModel
from ..models.speech_req_response_format import SpeechReqResponseFormat
from ..models.speech_req_voice import SpeechReqVoice
from ..types import UNSET, Unset

T = TypeVar("T", bound="SpeechReq")


@_attrs_define
class SpeechReq:
    """
    Attributes:
        input_ (str):
        model (SpeechReqModel | Unset):  Default: SpeechReqModel.TTS_1_HD.
        voice (SpeechReqVoice | Unset):  Default: SpeechReqVoice.ALLOY.
        response_format (SpeechReqResponseFormat | Unset):  Default: SpeechReqResponseFormat.MP3.
        speed (float | Unset):  Default: 1.0.
    """

    input_: str
    model: SpeechReqModel | Unset = SpeechReqModel.TTS_1_HD
    voice: SpeechReqVoice | Unset = SpeechReqVoice.ALLOY
    response_format: SpeechReqResponseFormat | Unset = SpeechReqResponseFormat.MP3
    speed: float | Unset = 1.0
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        input_ = self.input_

        model: str | Unset = UNSET
        if not isinstance(self.model, Unset):
            model = self.model.value

        voice: str | Unset = UNSET
        if not isinstance(self.voice, Unset):
            voice = self.voice.value

        response_format: str | Unset = UNSET
        if not isinstance(self.response_format, Unset):
            response_format = self.response_format.value

        speed = self.speed

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "input": input_,
            }
        )
        if model is not UNSET:
            field_dict["model"] = model
        if voice is not UNSET:
            field_dict["voice"] = voice
        if response_format is not UNSET:
            field_dict["response_format"] = response_format
        if speed is not UNSET:
            field_dict["speed"] = speed

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        input_ = d.pop("input")

        _model = d.pop("model", UNSET)
        model: SpeechReqModel | Unset
        if isinstance(_model, Unset):
            model = UNSET
        else:
            model = SpeechReqModel(_model)

        _voice = d.pop("voice", UNSET)
        voice: SpeechReqVoice | Unset
        if isinstance(_voice, Unset):
            voice = UNSET
        else:
            voice = SpeechReqVoice(_voice)

        _response_format = d.pop("response_format", UNSET)
        response_format: SpeechReqResponseFormat | Unset
        if isinstance(_response_format, Unset):
            response_format = UNSET
        else:
            response_format = SpeechReqResponseFormat(_response_format)

        speed = d.pop("speed", UNSET)

        speech_req = cls(
            input_=input_,
            model=model,
            voice=voice,
            response_format=response_format,
            speed=speed,
        )

        speech_req.additional_properties = d
        return speech_req

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, cast

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.http_validation_error import HTTPValidationError
from ...types import UNSET, Response


def _get_kwargs(
    *,
    object_prefix: str,
) -> dict[str, Any]:

    params: dict[str, Any] = {}

    params["object_prefix"] = object_prefix

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/v1/files/s3_files",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> HTTPValidationError | list[str] | None | None:
    if response.status_code == 200:

        def _parse_response_200(data: object) -> list[str] | None:
            if data is None:
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                response_200_type_0 = cast(list[str], data)

                return response_200_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[str] | None, data)

        response_200 = _parse_response_200(response.json())

        return response_200

    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Response[HTTPValidationError | list[str] | None]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    object_prefix: str,
) -> Response[HTTPValidationError | list[str] | None]:
    """Get Files S3 Async

    Args:
        object_prefix (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[HTTPValidationError | list[str] | None]
    """

    kwargs = _get_kwargs(
        object_prefix=object_prefix,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    object_prefix: str,
) -> HTTPValidationError | list[str] | None | None:
    """Get Files S3 Async

    Args:
        object_prefix (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        HTTPValidationError | list[str] | None
    """

    return sync_detailed(
        client=client,
        object_prefix=object_prefix,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    object_prefix: str,
) -> Response[HTTPValidationError | list[str] | None]:
    """Get Files S3 Async

    Args:
        object_prefix (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[HTTPValidationError | list[str] | None]
    """

    kwargs = _get_kwargs(
        object_prefix=object_prefix,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    object_prefix: str,
) -> HTTPValidationError | list[str] | None | None:
    """Get Files S3 Async

    Args:
        object_prefix (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        HTTPValidationError | list[str] | None
    """

    return (
        await asyncio_detailed(
            client=client,
            object_prefix=object_prefix,
        )
    ).parsed



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.database_enum import DatabaseEnum
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.database_creds_model_credentials_info_type_0 import DatabaseCredsModelCredentialsInfoType0


T = TypeVar("T", bound="DatabaseCredsModel")


@_attrs_define
class DatabaseCredsModel:
    """Credentials model for database connections

    Attributes:
        database_type (DatabaseEnum):
        host (None | str | Unset):
        port (int | None | Unset):
        database (None | str | Unset):
        username (None | str | Unset):
        password (None | str | Unset):
        name (None | str | Unset):
        project_id (None | str | Unset):
        dataset (None | str | Unset):
        credentials_info (DatabaseCredsModelCredentialsInfoType0 | None | Unset):
        cloud_sql_instance_connection_name (None | str | Unset):
        cloud_sql_use_private_ip (bool | Unset):  Default: False.
        cloud_sql_enable_iam_auth (bool | Unset):  Default: False.
    """

    database_type: DatabaseEnum
    host: None | str | Unset = UNSET
    port: int | None | Unset = UNSET
    database: None | str | Unset = UNSET
    username: None | str | Unset = UNSET
    password: None | str | Unset = UNSET
    name: None | str | Unset = UNSET
    project_id: None | str | Unset = UNSET
    dataset: None | str | Unset = UNSET
    credentials_info: DatabaseCredsModelCredentialsInfoType0 | None | Unset = UNSET
    cloud_sql_instance_connection_name: None | str | Unset = UNSET
    cloud_sql_use_private_ip: bool | Unset = False
    cloud_sql_enable_iam_auth: bool | Unset = False
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.database_creds_model_credentials_info_type_0 import DatabaseCredsModelCredentialsInfoType0

        database_type = self.database_type.value

        host: None | str | Unset
        if isinstance(self.host, Unset):
            host = UNSET
        else:
            host = self.host

        port: int | None | Unset
        if isinstance(self.port, Unset):
            port = UNSET
        else:
            port = self.port

        database: None | str | Unset
        if isinstance(self.database, Unset):
            database = UNSET
        else:
            database = self.database

        username: None | str | Unset
        if isinstance(self.username, Unset):
            username = UNSET
        else:
            username = self.username

        password: None | str | Unset
        if isinstance(self.password, Unset):
            password = UNSET
        else:
            password = self.password

        name: None | str | Unset
        if isinstance(self.name, Unset):
            name = UNSET
        else:
            name = self.name

        project_id: None | str | Unset
        if isinstance(self.project_id, Unset):
            project_id = UNSET
        else:
            project_id = self.project_id

        dataset: None | str | Unset
        if isinstance(self.dataset, Unset):
            dataset = UNSET
        else:
            dataset = self.dataset

        credentials_info: dict[str, Any] | None | Unset
        if isinstance(self.credentials_info, Unset):
            credentials_info = UNSET
        elif isinstance(self.credentials_info, DatabaseCredsModelCredentialsInfoType0):
            credentials_info = self.credentials_info.to_dict()
        else:
            credentials_info = self.credentials_info

        cloud_sql_instance_connection_name: None | str | Unset
        if isinstance(self.cloud_sql_instance_connection_name, Unset):
            cloud_sql_instance_connection_name = UNSET
        else:
            cloud_sql_instance_connection_name = self.cloud_sql_instance_connection_name

        cloud_sql_use_private_ip = self.cloud_sql_use_private_ip

        cloud_sql_enable_iam_auth = self.cloud_sql_enable_iam_auth

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "database_type": database_type,
            }
        )
        if host is not UNSET:
            field_dict["host"] = host
        if port is not UNSET:
            field_dict["port"] = port
        if database is not UNSET:
            field_dict["database"] = database
        if username is not UNSET:
            field_dict["username"] = username
        if password is not UNSET:
            field_dict["password"] = password
        if name is not UNSET:
            field_dict["name"] = name
        if project_id is not UNSET:
            field_dict["project_id"] = project_id
        if dataset is not UNSET:
            field_dict["dataset"] = dataset
        if credentials_info is not UNSET:
            field_dict["credentials_info"] = credentials_info
        if cloud_sql_instance_connection_name is not UNSET:
            field_dict["cloud_sql_instance_connection_name"] = cloud_sql_instance_connection_name
        if cloud_sql_use_private_ip is not UNSET:
            field_dict["cloud_sql_use_private_ip"] = cloud_sql_use_private_ip
        if cloud_sql_enable_iam_auth is not UNSET:
            field_dict["cloud_sql_enable_iam_auth"] = cloud_sql_enable_iam_auth

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.database_creds_model_credentials_info_type_0 import DatabaseCredsModelCredentialsInfoType0

        d = dict(src_dict)
        database_type = DatabaseEnum(d.pop("database_type"))

        def _parse_host(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        host = _parse_host(d.pop("host", UNSET))

        def _parse_port(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        port = _parse_port(d.pop("port", UNSET))

        def _parse_database(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        database = _parse_database(d.pop("database", UNSET))

        def _parse_username(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        username = _parse_username(d.pop("username", UNSET))

        def _parse_password(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        password = _parse_password(d.pop("password", UNSET))

        def _parse_name(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        name = _parse_name(d.pop("name", UNSET))

        def _parse_project_id(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        project_id = _parse_project_id(d.pop("project_id", UNSET))

        def _parse_dataset(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        dataset = _parse_dataset(d.pop("dataset", UNSET))

        def _parse_credentials_info(data: object) -> DatabaseCredsModelCredentialsInfoType0 | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                credentials_info_type_0 = DatabaseCredsModelCredentialsInfoType0.from_dict(data)

                return credentials_info_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(DatabaseCredsModelCredentialsInfoType0 | None | Unset, data)

        credentials_info = _parse_credentials_info(d.pop("credentials_info", UNSET))

        def _parse_cloud_sql_instance_connection_name(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        cloud_sql_instance_connection_name = _parse_cloud_sql_instance_connection_name(
            d.pop("cloud_sql_instance_connection_name", UNSET)
        )

        cloud_sql_use_private_ip = d.pop("cloud_sql_use_private_ip", UNSET)

        cloud_sql_enable_iam_auth = d.pop("cloud_sql_enable_iam_auth", UNSET)

        database_creds_model = cls(
            database_type=database_type,
            host=host,
            port=port,
            database=database,
            username=username,
            password=password,
            name=name,
            project_id=project_id,
            dataset=dataset,
            credentials_info=credentials_info,
            cloud_sql_instance_connection_name=cloud_sql_instance_connection_name,
            cloud_sql_use_private_ip=cloud_sql_use_private_ip,
            cloud_sql_enable_iam_auth=cloud_sql_enable_iam_auth,
        )

        database_creds_model.additional_properties = d
        return database_creds_model

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="UserAttributes")


@_attrs_define
class UserAttributes:
    """
    Attributes:
        email (str | Unset):
        phone (str | Unset):
        password (str | Unset):
        data (Any | Unset):
        nonce (str | Unset):
    """

    email: str | Unset = UNSET
    phone: str | Unset = UNSET
    password: str | Unset = UNSET
    data: Any | Unset = UNSET
    nonce: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        email = self.email

        phone = self.phone

        password = self.password

        data = self.data

        nonce = self.nonce

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if email is not UNSET:
            field_dict["email"] = email
        if phone is not UNSET:
            field_dict["phone"] = phone
        if password is not UNSET:
            field_dict["password"] = password
        if data is not UNSET:
            field_dict["data"] = data
        if nonce is not UNSET:
            field_dict["nonce"] = nonce

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        email = d.pop("email", UNSET)

        phone = d.pop("phone", UNSET)

        password = d.pop("password", UNSET)

        data = d.pop("data", UNSET)

        nonce = d.pop("nonce", UNSET)

        user_attributes = cls(
            email=email,
            phone=phone,
            password=password,
            data=data,
            nonce=nonce,
        )

        user_attributes.additional_properties = d
        return user_attributes

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.knowledge_base_role import KnowledgeBaseRole
from ..types import UNSET, Unset

T = TypeVar("T", bound="KnowledgeBaseResponse")


@_attrs_define
class KnowledgeBaseResponse:
    """KB enriched with stats for UI lists.

    Attributes:
        field_id (str):
        name (str):
        owner_user_id (str):
        tenant_id (str):
        current_user_role (KnowledgeBaseRole): Role within a knowledge base / workspace.
        description (None | str | Unset):
        created_at (datetime.datetime | Unset):
        updated_at (datetime.datetime | Unset):
        file_count (int | Unset):  Default: 0.
        member_count (int | Unset):  Default: 0.
    """

    field_id: str
    name: str
    owner_user_id: str
    tenant_id: str
    current_user_role: KnowledgeBaseRole
    description: None | str | Unset = UNSET
    created_at: datetime.datetime | Unset = UNSET
    updated_at: datetime.datetime | Unset = UNSET
    file_count: int | Unset = 0
    member_count: int | Unset = 0
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        field_id = self.field_id

        name = self.name

        owner_user_id = self.owner_user_id

        tenant_id = self.tenant_id

        current_user_role = self.current_user_role.value

        description: None | str | Unset
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description

        created_at: str | Unset = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        updated_at: str | Unset = UNSET
        if not isinstance(self.updated_at, Unset):
            updated_at = self.updated_at.isoformat()

        file_count = self.file_count

        member_count = self.member_count

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "_id": field_id,
                "name": name,
                "owner_user_id": owner_user_id,
                "tenant_id": tenant_id,
                "current_user_role": current_user_role,
            }
        )
        if description is not UNSET:
            field_dict["description"] = description
        if created_at is not UNSET:
            field_dict["created_at"] = created_at
        if updated_at is not UNSET:
            field_dict["updated_at"] = updated_at
        if file_count is not UNSET:
            field_dict["file_count"] = file_count
        if member_count is not UNSET:
            field_dict["member_count"] = member_count

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        field_id = d.pop("_id")

        name = d.pop("name")

        owner_user_id = d.pop("owner_user_id")

        tenant_id = d.pop("tenant_id")

        current_user_role = KnowledgeBaseRole(d.pop("current_user_role"))

        def _parse_description(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        description = _parse_description(d.pop("description", UNSET))

        _created_at = d.pop("created_at", UNSET)
        created_at: datetime.datetime | Unset
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        _updated_at = d.pop("updated_at", UNSET)
        updated_at: datetime.datetime | Unset
        if isinstance(_updated_at, Unset):
            updated_at = UNSET
        else:
            updated_at = isoparse(_updated_at)

        file_count = d.pop("file_count", UNSET)

        member_count = d.pop("member_count", UNSET)

        knowledge_base_response = cls(
            field_id=field_id,
            name=name,
            owner_user_id=owner_user_id,
            tenant_id=tenant_id,
            current_user_role=current_user_role,
            description=description,
            created_at=created_at,
            updated_at=updated_at,
            file_count=file_count,
            member_count=member_count,
        )

        knowledge_base_response.additional_properties = d
        return knowledge_base_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class LastErrorCode(str, Enum):
    INVALID_FILE = "invalid_file"
    SERVER_ERROR = "server_error"
    UNSUPPORTED_FILE = "unsupported_file"

    def __str__(self) -> str:
        return str(self.value)
